package jp.ac.hokudai.ist.main.aiwww.stlics.solver;

import jp.ac.hokudai.ist.main.aiwww.stlics.problem.CrispProblem;
import jp.ac.hokudai.ist.main.aiwww.stlics.solver.trait.CrispSolver;

/**
 * クリスプな制約充足問題のソルバを表す抽象クラスです．
 * ソルバに共通の動作(デバッグ)表示を提供します．
 * @author Takuto YANAGIDA
 * @version 2009/09/16
 */
public abstract class AbstractCrispSolver extends DebugSolver implements Solver, CrispSolver {

	/**
     * このソルバが対象とするクリスプな制約充足問題です．
     */
	transient protected CrispProblem pro_;

    /**
     * クリスプな制約充足問題を指定して，ソルバを生成します．
     * @param pro クリスプな制約充足問題
     */
	public AbstractCrispSolver(final CrispProblem pro) {
		super();
		pro_ = pro;
	}

	/**
	 * 各種アルゴリズムを実装するためのプレースホルダです．
	 * solveメソッドはこのメソッドを呼び出し，このメソッドの戻り値を返します．
	 * @return アルゴリズムが成功したらtrue，失敗したらfalse．
	 */
	abstract protected boolean exec();

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean solve() {
		double scr = 0.0;
		int uvs = 0;
		if(debug) {
			scr = pro_.satisfiedConstraintRate();
			uvs = pro_.emptyVariableSize();
			debugStream.println();
			debugStream.println("---- " + name() + " started ----");
		}
		final boolean res = exec();
		if(debug) {
			debugStream.println("Result: " + (res ? "success" : "failure"));
			debugStream.println("Satisfied Constraint Rate: " + scr + " -> " + pro_.satisfiedConstraintRate());
			debugStream.println("Remain violation: " + pro_.violatingConstraintSize());
			debugStream.println("Unassigned Size: " + uvs + " -> " + pro_.emptyVariableSize());
			debugStream.println("---- " + name() + " finished ----");
			debugStream.println();
		}
		return res;
	}

}
