package jp.ac.hokudai.ist.main.aiwww.stlics.util;

import jp.ac.hokudai.ist.main.aiwww.stlics.problem.Variable;

/**
 * 変数とそれに割り当てる値の組を表現するクラスです．
 * @author Takuto YANAGIDA
 * @version 2010/12/02
 */
public class Assignment {

	private final Variable variable_;
	private final int value_;

	/**
	 * 既存の割り当てのコピーを生成します．
	 * @param assignment 既存の割り当て
	 */
	public Assignment(final Assignment assignment) {
		this(assignment.variable(), assignment.value());
	}

	/**
	 * 変数を指定して割り当てを生成します．
	 * 現在の変数の値が保存されます．
	 * @param var 保存する変数
	 */
	public Assignment(final Variable var) {
		variable_ = var;
		value_ = var.value();
	}

	/**
	 * 変数と値を指定して割り当てを生成します．
	 * @param variable 保存する変数
	 * @param value 保存する値
	 */
	public Assignment(final Variable variable, final int value) {
		variable_ = variable;
		value_ = value;
	}

	/**
	 * 保存している変数に値を割り当てます．
	 */
	public void apply() {
		variable_.assign(value_);
	}

	/**
	 * 文字列表現を返します．
	 * @return 文字列表現
	 */
	@Override
	public String toString() {
		return "v" + variable_.index() + " <- " + value_;
	}

	/**
	 * 値を取得します．
	 * @return 値
	 */
	public int value() {
		return value_;
	}

	/**
	 * 変数を取得します．
	 * @return 変数
	 */
	public Variable variable() {
		return variable_;
	}

}
